// Soubor SHODA2.CPP
// Implementaace sablony zasobniku, ktera ukazuje,
// jak to vypada v pripade vyjimek.
// Metody vloz() a vyjmi() jsou
// implementovany velmi naivne, pri slutecnem pouziti
// by to bylo treba udelat lepe.
#include <stdexcept>
#include <algorithm>
#include <memory>
using namespace std;
// Naivni implementace zasobniku

template<typename T>
class zasobnik
{
	int delka;						// Maximalni pocet prvku
	T * p;							// Ukazatel na pole s daty
	int vrchol;						// Index vrcholu zasobniku
public:
	explicit zasobnik(int d=10);	// Explicitni konstruktor
	zasobnik(zasobnik&);			// Kopirovaci konstruktor
	virtual ~zasobnik();
	T& operator[](int);				// Pristup k prvkum
	zasobnik& operator=(zasobnik&);	// Prirazovaci operator
	void vloz(const T& x);
	T vyjmi();
	void swap(zasobnik &dalsi);
};


template<typename T>
void zasobnik<T>::swap(zasobnik<T> &dalsi)
{
	::swap(p, dalsi.p);
	::swap(delka, dalsi.delka);
	::swap(vrchol, dalsi.vrchol);
}

template<typename T>
zasobnik<T>::zasobnik(int d)
{
	delka = d > 0 ? d : 10;
	vrchol = 0;					// Prazdny zasobnik;
	p = new T [delka];
}

template<typename T>
zasobnik<T>& zasobnik<T>::operator=(zasobnik<T>& q)
{
/*	// Neni osetreno proti vyjimkan
    if(dynamic_cast<void*>(this) != dynamic_cast<void*>(&q))
	{
		delka = q.delka;
		vrchol = q.vrchol;
		delete [] p;
		p = new T[delka];
		for(int i = 0; i < vrchol; i++)
			p[i] = q.p[i];
	}
*/
/*	// Osetreno, ale ne dobre - muze dojit k uniku pameti
	T* pom = new T[q.delka];
	for(int i = 0; i < q.vrchol; i++) pom[i] = q.p[i];
	swap(pom, p);
	delka = q.delka;
	vrchol = q.vrchol;
	delete [] pom;
*/
	zasobnik pom(q);
	swap(pom);
	return *this;
}

template<typename T>
zasobnik<T>::~zasobnik()
{
	delete[] p;
}

template<typename T>
T& zasobnik<T>::operator[](int i)
{
	return p[i];
}

template<typename T>
void zasobnik<T>::vloz(const T& co)
{							// Nedobra implementace
	if (vrchol < delka)
		p[vrchol++] = co;
	else throw runtime_error("preplneny zasobnik");
}

template<typename T>
T zasobnik<T>::vyjmi()
{
	 return p[--vrchol];
}

template<typename T>
zasobnik<T>::zasobnik(zasobnik<T>& dalsi)
:delka(dalsi.delka), vrchol(0), p(0)
{
	try {
            p = new T[delka];
	    for(int i = 0; i < dalsi.vrchol; i++) vloz(dalsi[i]);
	}
        catch(...)
        {
          delete p;
        }
}


int main()
{
	try{
        zasobnik<int> z1;
	zasobnik<int> z2;
	z2.vloz(4);
	z2.vloz(8);
	z2.vloz(9);
    	z1 = z2;
	int x = z1.vyjmi();
	z1 = z1;
	}
        catch(...,)
        {
        }
}
