#include <iostream>
#include <deque>
#include <vector>
#include <algorithm>
using namespace std;

// barvy pro kresleni grafickych objektu a jejich nazvy
enum Barva {cerna, bila, zelena, modra, cervena};
char *BARVA[] = {"cerna", "bila", "zelena", "modra", "cervena"};

class GO								// Spolecny predek vsech grafickych objektu
{
protected:
   int barva;
public:
   GO(int barva): barva(barva) {}
   int &Barva() {return barva;}
   void Zobraz() const;
   void Smaz() const;
   virtual void Nakresli(int _barva) const = 0;
   virtual ~GO(){}
};

void GO::Zobraz() const
{
   Nakresli(barva);
}
void GO::Smaz() const
{
   Nakresli(cerna);
}
/*
void GO::Nakresli(int _barva) const
{
   cout << "GO, barva " << BARVA[_barva] << endl;
}
*/
class bod: public GO
{
protected:
   int x, y;
public:
   bod(int x, int y, int barva): GO(barva), x(x), y(y) {}
   void Nakresli(int _barva) const;
   ~bod(){cout << "d" << endl;}
};

void bod::Nakresli(int _barva) const
{
   cout	<< "bod, barva " << BARVA[_barva] << ", souradnice " 
   << x << ", " << y << endl;
}
class usecka: public GO { 
bod A, B;
public:
   usecka(int x1, int y1, int x2, int y2, int barva)
   : GO(barva), A(x1, y1, barva), B(x2, y2, barva)
   {}
   void Nakresli(int _barva) const;
};

void usecka::Nakresli(int _barva)const 
{
   cout << "usecka, barva " << BARVA[_barva] << ", pocatek a konec: "; 
   A.Nakresli(_barva);
   B.Nakresli(_barva);
}

class obrazek: public GO
{
	deque<GO*> obrazky;
public:
	obrazek():GO(0){}
	void Pridej(GO* g);
	void Nakresli(int _barva) const;
	void Vyprazdni();
};

void obrazek::Vyprazdni()
{
	for(int i = 0; i < obrazky.size(); i++)
		delete obrazky[i];
}

void obrazek::Pridej(GO* g)
{
	obrazky.push_back(g);
}
 
void obrazek::Nakresli(int barva) const
{
	deque<GO*>::const_iterator i = obrazky.begin();
	while(i != obrazky.end())
	{
		(*i++)->Nakresli(barva);
	}
}

class PrycSNim
{
	int barva;
public:
	PrycSNim(int barva): barva(barva){}
	bool operator()(GO* q)
	{
		return q->Barva() == barva;
	}
};

class Kompozice
{
	deque<GO*> obr;
public:
	Kompozice(){}
	void Pridej(GO* g);
	void Zobraz();
	void Cenzura(int barva);
};

void Kompozice::Pridej(GO* g)
{
	obr.push_back(g);
}

void Kompozice::Zobraz()
{
	for(int i = 0; i < obr.size(); i++)
		obr[i]->Zobraz();
}

/*
void Kompozice::Cenzura(int barva)
{
	obr.erase(remove_if(obr.begin(), obr.end(), PrycSNim(barva)), obr.end());
}
*/

void Kompozice::Cenzura(int barva)
{
	deque<GO*>::iterator i = remove_if(obr.begin(), obr.end(), PrycSNim(modra));
	obr.erase(i, obr.end());
}

int main()
{
	Kompozice kresba;
	kresba.Pridej(new bod(23, 54, modra));
	kresba.Pridej(new bod(53, 34, zelena));
	kresba.Pridej(new bod(21, 66, modra));
	kresba.Pridej(new bod(76, 55, bila));
	kresba.Zobraz();
	kresba.Cenzura(modra);
	kresba.Zobraz();

}