// Oprava chybne hierarchie -- "logicka dedicnost"

#include <iostream>
#include <string>
#include <cassert>
using namespace std;

class GO			// Graficky objekt
{
public:
	enum barvy{cerna, bila};
	static string nazvy[];
	// abstraktni metody
	virtual void nakresli(int barva) = 0;
	virtual void posun(int dx, int dy)=0;
	// nevirtualni metody (potomci je neprekryvaji)
	void zobraz(){nakresli(bila);}
	void smaz(){nakresli(cerna);}
};

string GO::nazvy[]={"cerna", "bila"};

class Bod: public GO
{
protected:
	int x, y;
public:
	Bod(int xx, int yy):x(xx), y(yy){}
	virtual void nakresli(int barva);
	virtual void posun(int dx, int dy);
	void zmenSouradnice(int dx, int dy){x += dx; y += dy;}
};

void Bod::posun(int dx, int dy)
{
	smaz();
	zmenSouradnice(dx, dy);
	zobraz();
}


void Bod::nakresli(int barva)
{
	cout << "Kreslim Bod, barva: " << nazvy[barva]
	     << " (" << x << ", " << y << ")" << endl; 
}

class Kruznice: public GO
{
	int r;
	Bod stred;
public:
	Kruznice(int xx, int yy, int rr): stred(xx, yy), r(rr){}
	virtual void nakresli(int barva);
	virtual void posun(int dx, int dy);
};

void Kruznice::posun(int dx, int dy)
{
	smaz();
	stred.zmenSouradnice(dx, dy);
	zobraz();
}

void Kruznice::nakresli(int barva)
{
	cout << "Kreslim Kruznici, barva: " << nazvy[barva] << ", stred: ";
	stred.nakresli(barva);
	cout << ", polomer: " << r << endl;
}

class Usecka: public GO
{					// Zdedeny podobjekt je pocatkem usecky
	Bod pocatek, konec;
public:
	Usecka(int x1, int y1, int x2, int y2)
		: pocatek(x1, y1), konec(x2, y2){}
	Usecka(Bod& A, Bod& B)
		: pocatek(A), konec(B){}
	virtual void nakresli(int barva);
	virtual void posun(int dx, int dy);
};


void Usecka::nakresli(int barva)
{
	cout << "Kreslim Usecku, barva: " << nazvy[barva] << ", pocatek a konec: " << endl;
	pocatek.nakresli(barva);
	konec.nakresli(barva);
}

void Usecka::posun(int dx, int dy)
{
	smaz();
	pocatek.zmenSouradnice(dx, dy);
	konec.zmenSouradnice(dx, dy);
	zobraz();
}

class Obdelnik: public GO
{
protected:
	Bod LDR;	// Levy dolni roh
	int vyska, sirka;
public:
	Obdelnik(int x, int y, int v, int s): LDR(x, y), vyska(v), sirka(s){}
	Obdelnik(Bod ldr, int v, int s): LDR(ldr), vyska(v), sirka(s){}
	virtual void nakresli(int barva);
	virtual void posun(int dx, int dy);
	void nastavVysku(int v) {vyska = v;}
	void nastavSirku(int v) {sirka = v;}
	int vratVysku(){return vyska;}
	int vratSirku(){return sirka;}
};

void Obdelnik::nakresli(int barva)
{
	cout << "Kreslim Obdlnik, barva: " << nazvy[barva] << ", Levy dolni roh: " << endl;
	LDR.nakresli(barva);
	cout << "vyska: " << vyska << ", sirka: "  << sirka << endl;
}

void Obdelnik::posun(int dx, int dy)
{
	smaz();
	LDR.zmenSouradnice(dx, dy);
	zobraz();
}

class Ctverec: public GO
{
protected:
	Bod LDR;	// Levy dolni roh
	int strana;
public:
	Ctverec(int x, int y, int v): LDR(x, y), strana(v) {}
	Ctverec(Bod ldr, int v): LDR(ldr), strana(v){}
	virtual void nakresli(int barva);
	virtual void posun(int dx, int dy);
	void nastavStranu(int v) {strana = v;}
	int vratStranu(){return strana;}
	operator Obdelnik(){return Obdelnik(LDR, strana, strana);}
};

void Ctverec::nakresli(int barva)
{
	cout << "Kreslim Ctverec, barva: " << nazvy[barva] << ", Levy dolni roh: " << endl;
	LDR.nakresli(barva);
	cout << "strana: " << strana << endl;
}

void Ctverec::posun(int dx, int dy)
{
	smaz();
	LDR.zmenSouradnice(dx, dy);
	zobraz();
}


void f(Obdelnik& obd)
{
	obd.nastavSirku(5);
	obd.nastavVysku(11);
	assert(obd.vratSirku() == 5);
}
int main()
{
	Ctverec c(1,2,3);
	//f(c);
	Obdelnik o(1,3,5,7);
	o.zobraz();
	o.posun(1,3);
	Kruznice K(3,4,5);
	K.zobraz();
	K.posun(1,1);
	Usecka U(1,2,3,4);
	U.zobraz();
	U.posun(4,6);
}
